; Leuchtmusterprogramm fuer AT 89 C 2051
; Es werden Zweifarb-LED's mit zwei Pins
; ueber eine Schieberegisterkette bestehend aus 6 Stck
; 74 HCT 164 angesteuert
; Quarzfrequenz 7,3728 MHz
;
	BSEG
P1X   EQU    97H  		;Takt BG1
P1Y   EQU    96H              ;Daten BG1
P2X   EQU    95H              ;Takt BG2
P2Y   EQU    94H              ;Daten BG2
P3X   EQU    93H		;Takt BG3
P3Y   EQU    92H		;Daten BG3
P4X   EQU    91H		;Takt BG4
P4Y   EQU    90H		;Daten BG4
;
	DSEG
B	EQU    0F0H		;B-Register
PSW    EQU    0D0H		;Flag Register
DPH    EQU    83H		;Data-Pointer high
DPL    EQU    82H		;Data-Pointer low
	CSEG
	ORG 0000
ANF:    ACALL LOE1
	MOV DPTR,#MUST
        MOV R5,#0D8H		;8 Sequenzen a 24 Byte
HP1:	CLR C			;Carry loeschen
	ACALL MUAU1		;Musterausgabe 1
	ACALL ZEIKO		;Korrektur Zeiger
	ACALL MUAU2		;Musterausgabe 2
	ACALL ZEIKO		;Korrektur Zeiger
	ACALL MUAU3		;Musterausgabe 3
	ACALL ZEIKO             ;Korrektur Zeiger
        ACALL MUAU4             ;Musterausgabe 4
	ACALL ZEIT		;Zeitschleife
        DJNZ R5,HP1             ;alle Sequenzen ?
	AJMP ANF		;
;
; Zeitschleife
;
ZEIT:   PUSH PSW
	MOV R3,#0FFH		;Zeitkonstante 1
ZK2:	MOV R4,#0FFH		;Zeitkonstante 2
ZK1:	NOP
	NOP
        NOP
	DJNZ R4,ZK1		;Schleife 1 fertig ?
	DJNZ R3,ZK2             ;Schleife 2 fertig ?
        POP PSW
	RET
;
; Loeschen der Anzeige 1
;
LOE1:   MOV B,#30H		;48 Schieberegzellen
        CLR P1Y		;Daten auf Low
LOE11:	ACALL TAKT1		;Taktsignal1
	DJNZ B,LOE11		;fertig ?
	RET
;
; Loeschen der Anzeige 2
;
LOE2:	MOV B,#30H		;48 Schieberegzellen
	CLR P2Y        	;Daten auf Low
LOE21:  ACALL TAKT2             ;Taktsignal2
	DJNZ B,LOE21		;fertig ?
	RET
;
; Zeigerkorrektur
;
ZEIKO:	CLR C			;Carry loeschen
	MOV A,DPL		;Low-Zeiger nach A
	SUBB A,#06H		;um 6 Stellen aendern
	MOV DPL,A		;wieder nach DPL
	JNC ZEIKO1		;falls kein Uebertrag
	DEC DPH			;wenn Uebertrag
ZEIKO1: RET			;erledigt
;
; Musterausgabe 1
;
MUAU1:  MOV R2,#06H		;6 Bytes fuer Kette
BYAUS1: MOV R1,#08H		;1 Byte = 8 Bit
	MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR          ;Byte holen
BAUS1: 	RRC A                   ;Ausgabebit nach Carry
	MOV P1Y,C		;ausgeben
	ACALL TAKT1		;Taktsignal
	DJNZ R1,BAUS1		;Byte fertig ?
	INC DPTR		;naechstes Byte
	DJNZ R2,BYAUS1		;Muster fertig
	RET
;
; Musterausgabe 2
;
MUAU2:  MOV R2,#06H		;6 Bytes fuer Kette
BYAUS2: MOV R1,#08H		;1 Byte = 8 Bit
	MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR		;Byte holen
BAUS2:  RRC A			;Ausgabebit nach Carry
	MOV P2Y,C		;ausgeben
	ACALL TAKT2		;Taktsignal
	DJNZ R1,BAUS2		;Byte fertig ?
	INC DPTR		;naechstes Byte
	DJNZ R2,BYAUS2		;Muster fertig ?
	RET
;
; Musterausgabe 3
;
MUAU3:  MOV R2,#06H		;6 Bytes fuer Kette
BYAUS3: MOV R1,#08H		;1 Byte = 8 Bit
	MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR		;Byte holen
BAUS3:  RRC A			;Ausgabebit nach Carry
	MOV P3Y,C		;ausgeben
	ACALL TAKT3		;Taktsignal
	DJNZ R1,BAUS3		;Byte fertig  ?
	INC DPTR		;naechstes Byte
	DJNZ R2,BYAUS3		;Muster fertig ?
	RET
;
; Musterausgabe 4
;
MUAU4:  MOV R2,#06H		;6 Bytes fuer Kette
BYAUS4: MOV R1,#08H		;1 Byte = 8 Bit
        MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR		;Byte holen
BAUS4:  RRC A                   ;Ausgabebit nach Carry
	MOV P4Y,C		;ausgeben
	ACALL TAKT4 		;Taktsignal
	DJNZ R1,BAUS4		;Byte fertig
	INC DPTR		;naechstes Byte
	DJNZ R2,BYAUS4		;Muster fertig ?
	RET
;
; Aktivierung des Taktsignals 1
;
TAKT1:  CLR P1X	        	;Takt auf Low
	SETB P1X		;Takt auf High
	RET
;
; Aktivierung des Taktsignals 2
;
TAKT2:  CLR P2X			;Takt auf Low
	SETB P2X		;Takt auf High
	RET
;
; Aktvierung des Taktsignals 3
;
TAKT3:  CLR P3X			;Takt auf Low
	SETB P3X                ;Takt auf High
	RET
;
; Aktivierung Taktsignal 4
;
TAKT4:	CLR P4X 		;Takt auf Low
	SETB P4X		;Takt auf High
	RET
;
; Punkt Farbe 1 und Farbe 2 abwechselnd von hinten nach vorn
MUST:
DB 80H, 00H, 00H, 00H, 00H, 00H
DB 80H, 00H, 00H, 40H, 00H, 00H
DB 0A0H, 00H, 00H, 40H, 00H, 00H
DB 0A0H, 00H, 00H, 50H, 00H, 00H
DB 0A8H, 00H, 00H, 50H, 00H, 00H
DB 0A8H, 00H, 00H, 54H, 00H, 00H
DB 0AAH, 00H, 00H, 54H, 00H, 00H
DB 0AAH, 00H, 00H, 55H, 00H, 00H
DB 0AAH, 80H, 00H, 55H, 00H, 00H
DB 0AAH, 80H, 00H, 55H, 40H, 00H
DB 0AAH, 0A0H, 00H, 55H, 40H, 00H
DB 0AAH, 0A0H, 00H, 55H, 50H, 00H
DB 0AAH, 0A8H, 00H, 55H, 50H, 00H
DB 0AAH, 0A8H, 00H, 55H, 54H, 00H
DB 0AAH, 0AAH, 00H, 55H, 54H, 00H
DB 0AAH, 0AAH, 00H, 55H, 55H, 00H
DB 0AAH, 0AAH, 80H, 55H, 55H, 00H
DB 0AAH, 0AAH, 80H, 55H, 55H, 40H
DB 0AAH, 0AAH, 0A0H, 55H, 55H, 40H
DB 0AAH, 0AAH, 0A0H, 55H, 55H, 50H
DB 0AAH, 0AAH, 0A8H, 55H, 55H, 50H
DB 0AAH, 0AAH, 0A8H, 55H, 55H, 54H
DB 0AAH, 0AAH, 0AAH, 55H, 55H, 54H
DB 0AAH, 0AAH, 0AAH, 55H, 55H, 55H
; Punkt Farbe 1 von vorn nach hinten
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FEH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FDH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FBH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0F7H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0EFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0DFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0BFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 7FH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FEH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FDH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FBH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0F7H, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0EFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0DFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0BFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 7FH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FEH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FDH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FBH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0F7H, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0EFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0DFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0BFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 7FH, 0FFH, 0FFH
; Farbe 2 von hinten nach vorn
DB 7FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0BFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0DFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0EFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0F7H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FBH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FDH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FEH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 7FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0BFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0DFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0EFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0F7H, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FBH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FDH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FEH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 7FH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0BFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0DFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0EFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0F7H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FBH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FDH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FEH, 0FFH, 0FFH, 0FFH
; Punkt Farbe 2 von vorn nach hinten
DB 00H, 00H, 00H, 00H, 00H, 01H
DB 00H, 00H, 00H, 00H, 00H, 02H
DB 00H, 00H, 00H, 00H, 00H, 04H
DB 00H, 00H, 00H, 00H, 00H, 08H
DB 00H, 00H, 00H, 00H, 00H, 10H
DB 00H, 00H, 00H, 00H, 00H, 20H
DB 00H, 00H, 00H, 00H, 00H, 40H
DB 00H, 00H, 00H, 00H, 00H, 80H
DB 00H, 00H, 00H, 00H, 01H, 00H
DB 00H, 00H, 00H, 00H, 02H, 00H
DB 00H, 00H, 00H, 00H, 04H, 00H
DB 00H, 00H, 00H, 00H, 08H, 00H
DB 00H, 00H, 00H, 00H, 10H, 00H
DB 00H, 00H, 00H, 00H, 20H, 00H
DB 00H, 00H, 00H, 00H, 40H, 00H
DB 00H, 00H, 00H, 00H, 80H, 00H
DB 00H, 00H, 00H, 01H, 00H, 00H
DB 00H, 00H, 00H, 02H, 00H, 00H
DB 00H, 00H, 00H, 04H, 00H, 00H
DB 00H, 00H, 00H, 08H, 00H, 00H
DB 00H, 00H, 00H, 10H, 00H, 00H
DB 00H, 00H, 00H, 20H, 00H, 00H
DB 00H, 00H, 00H, 40H, 00H, 00H
DB 00H, 00H, 00H, 80H, 00H, 00H
; Farbe 1 nach vorn akkumulierend
DB 80H, 00H, 00H, 00H, 00H, 00H
DB 0C0H, 00H, 00H, 00H, 00H, 00H
DB 0E0H, 00H, 00H, 00H, 00H, 00H
DB 0F0H, 00H, 00H, 00H, 00H, 00H
DB 0F8H, 00H, 00H, 00H, 00H, 00H
DB 0FCH, 00H, 00H, 00H, 00H, 00H
DB 0FEH, 00H, 00H, 00H, 00H, 00H
DB 0FFH, 00H, 00H, 00H, 00H, 00H
DB 0FFH, 80H, 00H, 00H, 00H, 00H
DB 0FFH, 0C0H, 00H, 00H, 00H, 00H
DB 0FFH, 0E0H, 00H, 00H, 00H, 00H
DB 0FFH, 0F0H, 00H, 00H, 00H, 00H
DB 0FFH, 0F8H, 00H, 00H, 00H, 00H
DB 0FFH, 0FCH, 00H, 00H, 00H, 00H
DB 0FFH, 0FEH, 00H, 00H, 00H, 00H
DB 0FFH, 0FFH, 00H, 00H, 00H, 00H
DB 0FFH, 0FFH, 80H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0C0H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0E0H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0F0H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0F8H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FCH, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FEH, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 00H, 00H, 00H
; Farbe 1 von vorn akkumulierend
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FEH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FCH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0F8H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0F0H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0E0H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0C0H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 80H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FEH, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FCH, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0F8H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0F0H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0E0H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0C0H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 80H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0FEH, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0FCH, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0F8H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0F0H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0E0H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0C0H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 80H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 00H, 00H, 00H
; Farbe 2 von hinten akkumulierend
DB 7FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 3FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 1FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 07H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 03H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 01H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 7FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 3FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 1FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 0FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 07H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 03H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 01H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 7FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 3FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 1FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 0FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 07H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 03H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 01H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 00H, 0FFH, 0FFH, 0FFH
;Farbe 2 von vorn akkumulierend
DB 0FFH, 0FFH, 0FEH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FCH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0F8H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0F0H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0E0H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0C0H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 80H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FEH, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FCH, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0F8H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0F0H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0E0H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0C0H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 80H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0FEH, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0FCH, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0F8H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0F0H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0E0H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0C0H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 80H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 00H, 0FFH, 0FFH, 0FFH
; von zwei Seiten aufeinander zu und auseinander
DB 80H, 00H, 01H, 00H, 00H, 00H
DB 0C0H, 00H, 03H, 00H, 00H, 00H
DB 0E0H, 00H, 07H, 00H, 00H, 00H
DB 0F0H, 00H, 0FH, 00H, 00H, 00H
DB 0F8H, 00H, 1FH, 00H, 00H, 00H
DB 0FCH, 00H, 3FH, 00H, 00H, 00H
DB 0FEH, 00H, 7FH, 00H, 00H, 00H
DB 0FFH, 00H, 0FFH, 00H, 00H, 00H
DB 0FFH, 81H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0C3H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0E7H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 00H, 00H, 00H
DB 0FFH, 0E7H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0C3H, 0FFH, 00H, 00H, 00H
DB 0FFH, 81H, 0FFH, 00H, 00H, 00H
DB 0FFH, 00H, 0FFH, 00H, 00H, 00H
DB 0FEH, 00H, 7FH, 00H, 00H, 00H
DB 0FCH, 00H, 3FH, 00H, 00H, 00H
DB 0F8H, 00H, 1FH, 00H, 00H, 00H
DB 0F0H, 00H, 0FH, 00H, 00H, 00H
DB 0E0H, 00H, 07H, 00H, 00H, 00H
DB 0C0H, 00H, 03H, 00H, 00H, 00H
DB 80H, 00H, 01H, 00H, 00H, 00H
DB 00H, 00H, 00H, 00H, 00H, 00H
DB 00H, 00H, 00H, 00H, 00H, 00H
;
  END

