; Light pattern generator program for AT 89C2051.
; Two bicolour LEDs are controlled from two AT pins
; via a shift register chain consisting of
; six 74HCT164's.
; Quartz crystal frequency: 7.3728MHz.
;
	BSEG
P1X   EQU    97H  		;Clock BG1
P1Y   EQU    96H              ;Data BG1
P2X   EQU    95H              ;Clock BG2
P2Y   EQU    94H              ;Data BG2
P3X   EQU    93H		;Clock BG3
P3Y   EQU    92H		;Data BG3
P4X   EQU    91H		;Clock BG4
P4Y   EQU    90H		;Data BG4
;
	DSEG
B	EQU    0F0H		;B-Register
PSW    EQU    0D0H		;Flag Register
DPH    EQU    83H		;Data Pointer high
DPL    EQU    82H		;Data Pointer low
	CSEG
	ORG 0000
ANF:    ACALL LOE1
	MOV DPTR,#PATT
        MOV R5,#0D8H		;8 Sequences of 24 Bytes each
HP1:	CLR C			;Clear Carry
	ACALL MUAU1		;Pattern output 1
	ACALL ZEIKO		;Correction pointer
	ACALL MUAU2		;Pattern output 2
	ACALL ZEIKO		;Correction pointer
	ACALL MUAU3		;Pattern output 3
	ACALL ZEIKO             ;Correction pointer
        ACALL MUAU4             ;Pattern output 4
	ACALL ZEIT		;Loop
        DJNZ R5,HP1             ;All sequences?
	AJMP ANF		;
;
; Loop
;
ZEIT:   PUSH PSW
	MOV R3,#0FFH		;Time constant 1
ZK2:	MOV R4,#0FFH		;Time constant 2
ZK1:	NOP
	NOP
        NOP
	DJNZ R4,ZK1		;Loop 1 done?
	DJNZ R3,ZK2             ;Loop 2 done?
        POP PSW
	RET
;
; Clear readout 1
;
LOE1:   MOV B,#30H		;48 shift reg cells
        CLR P1Y		;Data to Low
LOE11:	ACALL TAKT1		;Clock signal1
	DJNZ B,LOE11		;done?
	RET
;
; Clear readout 2
;
LOE2:	MOV B,#30H		; 48 shift register cells 
	CLR P2Y        	;Data to Low
LOE21:  ACALL TAKT2             ;Clock signal 2
	DJNZ B,LOE21		;Done?
	RET
;
; Pointer correction
;
ZEIKO:	CLR C			;Clear Carry
	MOV A,DPL		;Low pointer to A
	SUBB A,#06H		;Change by 6 positions
	MOV DPL,A		;Again to DPL
	JNC ZEIKO1		;If no carry
	DEC DPH			;If carry
ZEIKO1: RET			;Done
;
; Pattern output 1
;
MUAU1:  MOV R2,#06H		;6 Bytes for chain
BYAUS1: MOV R1,#08H		;1 Byte = 8 Bit
	MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR          ;Fetch Byte
BAUS1: 	RRC A                   ;Output bit to Carry
	MOV P1Y,C		;Output it
	ACALL TAKT1		;Clock signal
	DJNZ R1,BAUS1		;Byte ready?
	INC DPTR		;Next Byte
	DJNZ R2,BYAUS1		;Pattern done?
	RET
;
; Pattern output 2
;
MUAU2:  MOV R2,#06H		;6 Bytes for chain
BYAUS2: MOV R1,#08H		;1 Byte = 8 Bit
	MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR		;Fetch Byte
BAUS2:  RRC A			;Output bit to Carry
	MOV P2Y,C		;Output it
	ACALL TAKT2		;Clock signal
	DJNZ R1,BAUS2		;Byte ready?
	INC DPTR		;Next Byte
	DJNZ R2,BYAUS2		;Pattern done?
	RET
;
; Pattern output 3
;
MUAU3:  MOV R2,#06H		;6 Bytes for chain 
BYAUS3: MOV R1,#08H		;1 Byte = 8 Bit
	MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR		;Fetch Byte
BAUS3:  RRC A			;Output bit to Carry
	MOV P3Y,C		;Output it
	ACALL TAKT3		;Clock signal
	DJNZ R1,BAUS3		;Byte ready?
	INC DPTR		;Next Byte
	DJNZ R2,BYAUS3		;Pattern done?
	RET
;
; Pattern output 4
;
MUAU4:  MOV R2,#06H		;6 Bytes for chain
BYAUS4: MOV R1,#08H		;1 Byte = 8 Bit
        MOV A,#00H		;Offset = 0
	MOVC A,@A+DPTR		;Fetch Byte
BAUS4:  RRC A                   ;Output bit to Carry
	MOV P4Y,C		;Output it
	ACALL TAKT4 		;Clock signal
	DJNZ R1,BAUS4		;Byte ready?
	INC DPTR		;Next Byte
	DJNZ R2,BYAUS4		;Pattern done?
	RET
;
; Clock signal 1 activation
;
TAKT1:  CLR P1X	        	;Clock at Low
	SETB P1X		;Clock at High
	RET
;
; Clock signal 2 activation
;
TAKT2:  CLR P2X			;Clock at Low
	SETB P2X		;Clock at High
	RET
;
; Clock signal 3 activation
;
TAKT3:  CLR P3X			;Clock at Low
	SETB P3X                ;Clock at High
	RET
;
; Clock signal 4 activation 
;
TAKT4:	CLR P4X 		;Clock at Low
	SETB P4X		;Clock at High
	RET
;
; Dot colour 1 and colour 2 alternately, back to front
PATT:
DB 80H, 00H, 00H, 00H, 00H, 00H
DB 80H, 00H, 00H, 40H, 00H, 00H
DB 0A0H, 00H, 00H, 40H, 00H, 00H
DB 0A0H, 00H, 00H, 50H, 00H, 00H
DB 0A8H, 00H, 00H, 50H, 00H, 00H
DB 0A8H, 00H, 00H, 54H, 00H, 00H
DB 0AAH, 00H, 00H, 54H, 00H, 00H
DB 0AAH, 00H, 00H, 55H, 00H, 00H
DB 0AAH, 80H, 00H, 55H, 00H, 00H
DB 0AAH, 80H, 00H, 55H, 40H, 00H
DB 0AAH, 0A0H, 00H, 55H, 40H, 00H
DB 0AAH, 0A0H, 00H, 55H, 50H, 00H
DB 0AAH, 0A8H, 00H, 55H, 50H, 00H
DB 0AAH, 0A8H, 00H, 55H, 54H, 00H
DB 0AAH, 0AAH, 00H, 55H, 54H, 00H
DB 0AAH, 0AAH, 00H, 55H, 55H, 00H
DB 0AAH, 0AAH, 80H, 55H, 55H, 00H
DB 0AAH, 0AAH, 80H, 55H, 55H, 40H
DB 0AAH, 0AAH, 0A0H, 55H, 55H, 40H
DB 0AAH, 0AAH, 0A0H, 55H, 55H, 50H
DB 0AAH, 0AAH, 0A8H, 55H, 55H, 50H
DB 0AAH, 0AAH, 0A8H, 55H, 55H, 54H
DB 0AAH, 0AAH, 0AAH, 55H, 55H, 54H
DB 0AAH, 0AAH, 0AAH, 55H, 55H, 55H
; Dot colour 1, front to back
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FEH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FDH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FBH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0F7H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0EFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0DFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0BFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 7FH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FEH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FDH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FBH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0F7H, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0EFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0DFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0BFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FFH, 7FH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FEH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FDH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0FBH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0F7H, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0EFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0DFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 0BFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FFH, 7FH, 0FFH, 0FFH
; Colour 2, back to front
DB 7FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0BFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0DFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0EFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0F7H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FBH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FDH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FEH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 7FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0BFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0DFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0EFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0F7H, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FBH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FDH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FEH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 7FH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0BFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0DFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0EFH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0F7H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FBH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FDH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FEH, 0FFH, 0FFH, 0FFH
; Dot colour 2, front to back
DB 00H, 00H, 00H, 00H, 00H, 01H
DB 00H, 00H, 00H, 00H, 00H, 02H
DB 00H, 00H, 00H, 00H, 00H, 04H
DB 00H, 00H, 00H, 00H, 00H, 08H
DB 00H, 00H, 00H, 00H, 00H, 10H
DB 00H, 00H, 00H, 00H, 00H, 20H
DB 00H, 00H, 00H, 00H, 00H, 40H
DB 00H, 00H, 00H, 00H, 00H, 80H
DB 00H, 00H, 00H, 00H, 01H, 00H
DB 00H, 00H, 00H, 00H, 02H, 00H
DB 00H, 00H, 00H, 00H, 04H, 00H
DB 00H, 00H, 00H, 00H, 08H, 00H
DB 00H, 00H, 00H, 00H, 10H, 00H
DB 00H, 00H, 00H, 00H, 20H, 00H
DB 00H, 00H, 00H, 00H, 40H, 00H
DB 00H, 00H, 00H, 00H, 80H, 00H
DB 00H, 00H, 00H, 01H, 00H, 00H
DB 00H, 00H, 00H, 02H, 00H, 00H
DB 00H, 00H, 00H, 04H, 00H, 00H
DB 00H, 00H, 00H, 08H, 00H, 00H
DB 00H, 00H, 00H, 10H, 00H, 00H
DB 00H, 00H, 00H, 20H, 00H, 00H
DB 00H, 00H, 00H, 40H, 00H, 00H
DB 00H, 00H, 00H, 80H, 00H, 00H
; Colour 1 to front, accumulating
DB 80H, 00H, 00H, 00H, 00H, 00H
DB 0C0H, 00H, 00H, 00H, 00H, 00H
DB 0E0H, 00H, 00H, 00H, 00H, 00H
DB 0F0H, 00H, 00H, 00H, 00H, 00H
DB 0F8H, 00H, 00H, 00H, 00H, 00H
DB 0FCH, 00H, 00H, 00H, 00H, 00H
DB 0FEH, 00H, 00H, 00H, 00H, 00H
DB 0FFH, 00H, 00H, 00H, 00H, 00H
DB 0FFH, 80H, 00H, 00H, 00H, 00H
DB 0FFH, 0C0H, 00H, 00H, 00H, 00H
DB 0FFH, 0E0H, 00H, 00H, 00H, 00H
DB 0FFH, 0F0H, 00H, 00H, 00H, 00H
DB 0FFH, 0F8H, 00H, 00H, 00H, 00H
DB 0FFH, 0FCH, 00H, 00H, 00H, 00H
DB 0FFH, 0FEH, 00H, 00H, 00H, 00H
DB 0FFH, 0FFH, 00H, 00H, 00H, 00H
DB 0FFH, 0FFH, 80H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0C0H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0E0H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0F0H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0F8H, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FCH, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FEH, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 00H, 00H, 00H
; Colour 1, accumulating from front
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FEH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0FCH
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0F8H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0F0H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0E0H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 0C0H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 80H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FFH, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FEH, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0FCH, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0F8H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0F0H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0E0H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 0C0H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 80H, 00H
DB 0FFH, 0FFH, 0FFH, 0FFH, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0FEH, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0FCH, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0F8H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0F0H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0E0H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 0C0H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 80H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 00H, 00H, 00H
; Colour 2, accumulating from back
DB 7FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 3FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 1FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 0FH, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 07H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 03H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 01H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 0FFH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 7FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 3FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 1FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 0FH, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 07H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 03H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 01H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 0FFH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 7FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 3FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 1FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 0FH, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 07H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 03H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 01H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 00H, 0FFH, 0FFH, 0FFH
; Colour 2, accumulating from front
DB 0FFH, 0FFH, 0FEH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0FCH, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0F8H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0F0H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0E0H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 0C0H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 80H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FFH, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FEH, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0FCH, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0F8H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0F0H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0E0H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 0C0H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 80H, 00H, 0FFH, 0FFH, 0FFH
DB 0FFH, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0FEH, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0FCH, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0F8H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0F0H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0E0H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 0C0H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 80H, 00H, 00H, 0FFH, 0FFH, 0FFH
DB 00H, 00H, 00H, 0FFH, 0FFH, 0FFH
; Approaching from 2 sides and parting again
DB 80H, 00H, 01H, 00H, 00H, 00H
DB 0C0H, 00H, 03H, 00H, 00H, 00H
DB 0E0H, 00H, 07H, 00H, 00H, 00H
DB 0F0H, 00H, 0FH, 00H, 00H, 00H
DB 0F8H, 00H, 1FH, 00H, 00H, 00H
DB 0FCH, 00H, 3FH, 00H, 00H, 00H
DB 0FEH, 00H, 7FH, 00H, 00H, 00H
DB 0FFH, 00H, 0FFH, 00H, 00H, 00H
DB 0FFH, 81H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0C3H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0E7H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0FFH, 0FFH, 00H, 00H, 00H
DB 0FFH, 0E7H, 0FFH, 00H, 00H, 00H
DB 0FFH, 0C3H, 0FFH, 00H, 00H, 00H
DB 0FFH, 81H, 0FFH, 00H, 00H, 00H
DB 0FFH, 00H, 0FFH, 00H, 00H, 00H
DB 0FEH, 00H, 7FH, 00H, 00H, 00H
DB 0FCH, 00H, 3FH, 00H, 00H, 00H
DB 0F8H, 00H, 1FH, 00H, 00H, 00H
DB 0F0H, 00H, 0FH, 00H, 00H, 00H
DB 0E0H, 00H, 07H, 00H, 00H, 00H
DB 0C0H, 00H, 03H, 00H, 00H, 00H
DB 80H, 00H, 01H, 00H, 00H, 00H
DB 00H, 00H, 00H, 00H, 00H, 00H
DB 00H, 00H, 00H, 00H, 00H, 00H
;
  END

